#include <reg51.h>
#include "d11_cmds.h"
#include "misc.h"
#include "d11chap9.h"
#include "io.h"
#define CONST_CLK_FREQ 12
#define CONST_CLK_DIVIDER (48/CONST_CLK_FREQ)-1
#define DEFAULT_ADDRESS 0x00

/* Register Defintinitions */
/*====================*/
data unsigned char CLK_DIVIDER;
/*--------------------*/
data char yRotation;
data char xRotation;
data unsigned char lock_write;

/* Function Prototype */
/*====================*/
void D11_initialise(void);
void USB_MAIN(void);
void Check_Mouse(void);
/*--------------------*/


void main(void)
{
	D11_initialise();
        while(1)
	{
		USB_MAIN();

                //------------------------------
                // Add your code here ..........
                //------------------------------
                Check_Mouse();
	}
}

void USB_MAIN(void)
{
	if (!D11_INT_)
	{
		UpdateIntReg();
		if (ENDPT_INT & CTRL_ENDPT_OUT)
		{ /* Control Endpoint OUT Interrupt */
			ProcessCtrlOutEP();
		} else
		if (ENDPT_INT & CTRL_ENDPT_IN)
		{ /* Control Endpoint IN Interrupt */
			ProcessCtrlInEP();
		} else
		if (ENDPT_INT & ENDPT1_IN)
		{ /* Endpoint 1 IN Interrupt */
	            ReadEndptStatus(ENDPT1_IN); //Clears the IN interrupt
                  lock_write = 0;
		} else
		if (OTHER_INT & BUS_RESET)
		{ /* Bus Reset Condition */
                  DisOtherAddress();
                  SetAddress(DEFAULT_ADDRESS | 0x80); //Set to Default Address
        		DEVICE_STATUS = 0X02; //REMOTE WAKEUP ENABLED, BUS-POWERED
			OPERATIONS_STATUS = 0x00; //Device not configured on Power up
			CLK_DIVIDER = CONST_CLK_DIVIDER; //Clk Divider is initialised here
			D11_SUSPEND = 0; //Preventd D11 to go into Suspend state
		}
	}
}

void D11_initialise(void)
{

        P0 = 0xFF;
        P1 = 0xFF;
        P2 = 0xFF;
        P3 = 0xFF;
        DEVICE_STATUS = 0X02; //REMOTE WAKEUP ENABLED, BUS-POWERED
	OPERATIONS_STATUS = 0x00; //Device not configured on Power up
	CLK_DIVIDER = CONST_CLK_DIVIDER; //Clk Divider is initialised here

	D11_SUSPEND = 0; //Prevents D11 to go into Suspend state
      DisOtherAddress();
      SetAddress(DEFAULT_ADDRESS | 0x80); //Set to Default Address
	//SetMode(0x85, CLK_DIVIDER);   //Disable softconnect
	//delay_1s();
	//delay_1s();                   //Wait till host can unhook
	//delay_1s();                   //device driver before re-enumerating
	//delay_1s();                   //again
	//delay_1s();
	SetMode(0x95, CLK_DIVIDER);   //Enable softconnect, Debug Mode for timing
}

void Check_Mouse(void)
{
unsigned char changed = 0;

   if (!UP_KEY)
   {
      yRotation++;
      changed = 1;
   } else
   if (!DOWN_KEY)   
   {
      yRotation--;
      changed = 1;
   }
   if (!RIGHT_KEY)
   {
      xRotation++;
      changed = 1;
   } else
   if (!LEFT_KEY)   
   {
      xRotation--;
      changed = 1;
   }

   if (changed & !lock_write)
   {
      DBUFFER[0] = 3; //SEND 3 BYTES TO HOST
      DBUFFER[1] = 0; //Click Status
      DBUFFER[2] = xRotation; //Horizontal Status
      DBUFFER[3] = yRotation; //Vertical Status
      yRotation = 0;
      xRotation = 0;
      SelectEndpoint(ENDPOINT1_IN);
	WriteBuffer();
      ValidateBuffer();
      lock_write = 1;
   }
}
