/*#pragma SRC*/
/* Filename : D11_RW.C                            */
/* Author   : Chui Wei Leong                      */
/*            Eric Lu                             */
/* File description : To provide read and write   */
/*                    interface to PDIUSBD11      */
/* Hardware Description :                         */
/*                                                */
/*                                                */
/* Driver portion to read/write from/to D11       */
/* Uses an Inline assembly instruction to do that */
/*================================================*/
#include <reg51.h>
#include <intrins.h>
#include <io.h>

/*================================================*/
#define I2C_DATA_R 0X35
#define I2C_DATA_W 0X34
#define I2C_COMMAND_W 0X36

/*================================================*/
/*REGISTERS ASSIGNMENTS*/
/*================================================*/
//extern unsigned char CLK_DIVIDER;
//char D11_RWDATA; //1 bytes of temporary storage
//     	         //data to swop between C and asm.

//==================================================


// I2C Routines
//*************

//==================================================
// Pulls SCL high, returns a 1 if successful else 0
//--------------------------------------------------
bit UI2cSCLHigh(void)
{
unsigned char cnt ;

	D11_SCL = 1;
	for (cnt=8;cnt>0;cnt--)
	{
		if (D11_SCL) return(1);
	};
	return(0);
}

bit UI2cStart(void)
{
	D11_SDA = 0 ;
        _nop_();
        _nop_();
	D11_SCL = 0 ;
	return(1) ;
}

bit UI2cStop(void)
{
	D11_SCL = 1;
        _nop_();
        _nop_();
        D11_SDA = 1 ;
	return(1);
}

//========================================================
// Returns a 1 if byte is successfully sent with Ack, else
// return zero, if either SCL is not clockable or no ack from
// I2C Device
//--------------------------------------------------------
bit UI2cSend(unsigned char SendData)
{
unsigned char mask;

        //Strobe first 8 bit of data with
        //MSB first to I2C bus
        //===============================
	mask = 0x80 ;
	do
	{
		D11_SDA = ((SendData & mask) != 0);
		mask = mask >>1 ;
		if(UI2cSCLHigh() == 0) return(0) ;
		D11_SCL = 0 ;
	}while (mask);

        //Check for number 9 acknowledgement bit from
        //I2C Device
        //===========================================
	D11_SDA =1 ;
        _nop_();
        _nop_();
        if(UI2cSCLHigh() == 0) return(0);
	if ( D11_SDA == 0 )
	{
              //Device has acknowledged
              _nop_();
              D11_SCL=0;
              return(1);
	};
	return(0);
}


bit UI2cReceive(unsigned char *pReceiveData, bit Ack)
{
unsigned char i,mask;
	D11_SDA = 1;
	mask = 0x80;
	i=0;
        _nop_();
	do
	{
		if(UI2cSCLHigh() == 0) return(0);
                _nop_();
                if (D11_SDA) i=i|mask ;
		D11_SCL = 0 ;
                _nop_();
                mask=mask>>1;
	}while(mask);

	if (Ack)
	  D11_SDA =1;
	else
	  D11_SDA =0;
	if(UI2cSCLHigh() == 0) return(0);
        _nop_();
        *pReceiveData=i;
	D11_SCL = 0;
	return(1);
}
//=================================================
void Wr_command(unsigned char D11_CDATA)
{
	UI2cStart(); //Single Master, ignores bus contention
	if (UI2cSend(I2C_COMMAND_W))
        {
           UI2cSend(D11_CDATA);
	   UI2cStop();
        }
        else
        {
                D11_SCL = 1;
                D11_SDA = 1;
        }
}

void Rd_Ndata(unsigned char *POINTER, unsigned char N)
{
unsigned char count;
bit ReceiveAck ;

	UI2cStart();  //Single Master, ignores bus contention
	if (UI2cSend(I2C_DATA_R))
        {
	     for (count = 0; count < N; count++)
	     {
		ReceiveAck= (count == (N-1));
                UI2cReceive(POINTER+count, ReceiveAck);
	     }
        }
	UI2cStop();
}

void Wr_Ndata(unsigned char *POINTER, unsigned char N)
{
unsigned char count;

	UI2cStart(); //Single Master, ignores bus contention
	if (UI2cSend(I2C_DATA_W))
        {
	   for (count = 0; count < N; count++)
	   {
		UI2cSend(*(POINTER+count));
	   }
	   UI2cStop();
        }
        else
        {
                D11_SCL = 1;
                D11_SDA = 1;
        }
}

void Rd_Bufdata(unsigned char *POINTER)
{
unsigned char count, numberBytes;
bit ReceiveAck ;

	UI2cStart();  //Single Master, ignores bus contention
	if (UI2cSend(I2C_DATA_R))
        {
             UI2cReceive(&count, 0); //Read reserve byte
             if (UI2cReceive(&numberBytes, 0)) //Read bytes available
             {
                *POINTER = numberBytes;
	        for (count = 1; count < (numberBytes+1); count++)
	        {
		   ReceiveAck= (count == numberBytes);
                   UI2cReceive(POINTER+count, ReceiveAck);
                }
	     }
        }
	UI2cStop();
}

void Wr_Bufdata(unsigned char *POINTER)
{
unsigned char count;

	UI2cStart(); //Single Master, ignores bus contention
	if (UI2cSend(I2C_DATA_W))
        {
           UI2cSend(0x00); //Reserved byte
	   for (count = 0; count < ((*POINTER)+1); count++)
	   {
		UI2cSend(*(POINTER+count));
	   }
	   UI2cStop();
        }
        else
        {
                D11_SCL = 1;
                D11_SDA = 1;
        }
}
