#ifndef SIO_MULTIPLAYER_H
#define SIO_MULTIPLAYER_H 1

//RCNT is used for mode selection
#define R_RCNT  *(volatile u16*) 0x4000134

#include <stdio.h>

/* GBAtek 1.4:
134h - RCNT (R) - SIO Mode, usage in GENERAL-PURPOSE Mode (R/W)
Interrupts can be requested when SI changes from HIGH to LOW, as General Purpose mode does
not require a serial shift clock, this interrupt may be produced even when the GBA is in
Stop (low power standby) state.

  Bit   Expl.
  0     SC Data Bit         (0=Low, 1=High)
  1     SD Data Bit         (0=Low, 1=High)
  2     SI Data Bit         (0=Low, 1=High)
  3     SO Data Bit         (0=Low, 1=High)
  4     SC Direction        (0=Input, 1=Output)
  5     SD Direction        (0=Input, 1=Output)
  6     SI Direction        (0=Input, 1=Output, but see below)
  7     SO Direction        (0=Input, 1=Output)
  8     Interrupt Request   (0=Disable, 1=Enable)
  9-13  Not used
  14    Must be "0" for General-Purpose Mode
  15    Must be "1" for General-Purpose or JOYBUS Mode

SI should be always used as Input to avoid problems with other hardware which does not
expect data to be output there.
*/

// Accept a string input of 0 and 1 chars
void set_comms(short SC, short SD, short SI, short SO, short IO){
    u16 data;
    data = (1*SC) + (2*SD) + (4*SI) + (8*SO);
    data = data + (IO * 0xf0);  // Set all pins to IO (1=Out, 0=In)
    data = data + 0x8000;       // Set general purpose mode ('10' MSB-LSB)

    R_RCNT = data;

    // Render the hex value into a string (as ham_DrawText does not support this)
    char st[10];
    sprintf(st,"[%04x]",R_RCNT);
    
    ham_DrawText(1,1, st);

    if(IO) ham_DrawText(1,2, "Output"); else ham_DrawText(1,2, "Input ");

    if(0x1 & R_RCNT) ham_DrawText(1,11,"SC"); else ham_DrawText(1,11,"--");
    if(0x2 & R_RCNT) ham_DrawText(5,11,"SD"); else ham_DrawText(5,11,"--");
    if(0x4 & R_RCNT) ham_DrawText(10,11,"SI"); else ham_DrawText(10,11,"--");
    if(0x8 & R_RCNT) ham_DrawText(15,11,"SO"); else ham_DrawText(15,11,"--");
}

#endif
