#include "text.h"
#include "zxfont.c" // Our imported font.

unsigned short tFGC; // Current foreground colour
unsigned short tBGC; // Current background colour
int CurX;            // Location to write next character (almost a cursor!)

void InitText(unsigned short ForeGround, unsigned short BackGround)
{
  // Set to mode 3
  *(unsigned int*)0x04000000 = 3 | (1<<10);
  SetScreenColour(BackGround); // Clear the screen
  tFGC = ForeGround;           // Store the text colours
  tBGC = BackGround;
  CurX = 0;                    // Reset the cursor
}

inline void cls(void)
{
  // Nice easy function to clear the screen
  SetScreenColour(tBGC);
  CurX = 0;
}

inline void ResetCursor(void)
{
  CurX = 0;
}

void Colour(unsigned short ForeGround, unsigned short BackGround)
{
  // Change the text colours. Much like the color command in QBasic
  tFGC = ForeGround;
  tBGC = BackGround;
}

void SetScreenColour(unsigned short Colour)
{
  // Wipe the screen by writing a certain colour to each pixel
  unsigned short *p; // Pixel Pointer

  // Loop throught the video buffer blanking each pixel
  for(p = (unsigned short*)0x06000000; p < (unsigned short *)0x06012C00; p++)
    *p = Colour;
}

void BltLetter(unsigned char c, int x, int y, unsigned short FColour, unsigned short BColour)
{
  // Draw a letter to the screen, not really a Blit at all
  unsigned short *p;     // The pixel we're writing
  unsigned short *check; // The memory location we are checking for
  unsigned char  BitBuffer;
  int yy, i;
 
  // Get memory location in video buffer
  p = (unsigned short*)0x06000000;
  p += ((y * 240) + x);

  i = c * 8; // Take this MUL out of the loop to speed it up
             // It's used to calculate the offset in the font array

  // Loop through each byte of the character bitmap
  for(yy = 0; yy < 8; yy++) {
    BitBuffer = gbfont[i + yy]; // Buffer the data as we'll be bit shifting it
    check = p + 8;              // Calculate the memory address to stop writing to the screen at

    // Loop through the video buffer
    for(; p < check; p++){
      // Write either the GBC or BGC depending on the most significant bit
      if(BitBuffer & 0x80) {
        *p = FColour;
      } else {
        *p = BColour;
      }
      // Bit shift the buffer to get the next pixel
      BitBuffer <<= 1;
    }
    // Move the pixel pointer down to the next row
    p += 232;
  }
}

void BltString(unsigned char * Text, int x, int y, unsigned short FColour, unsigned short BColour)
{
  // Write a string to the screen by calling BltLetter() for each character
  int count = 0;       // String Location counter
  if (y > 152) return; // Make sure we don't draw off the bottom of the screen

  // Keep looping until we hit a null zero
  while(Text[count] != 0) {
    // Make sure we're not to close to the edge of the screen
    if (x > 232) return;
    // Draw the character to the screen
    BltLetter(Text[count], x, y, FColour, BColour);
    // Move along 8 pixels
    x += 8;
    // Move to the next character
    count++;
  }
}

void ShiftScreen(void)
{
  // Simulate a scolling text console by shifting the screen up one line.
  // Work by simply copying memory locations in the video buffer. Nothing fancy
  unsigned short *p;

  // Move up the last 19 lines of text
  for(p = (unsigned short *)0x06000000; p < (unsigned short*)0x06011D00; p++)
    *p = *(unsigned short *)(p + 1920);

  // Blank out the last line of text
  for(;p < (unsigned short*)0x06012C00; p++)
    *p = tBGC;
}

void Print(unsigned char * Text)
{
  // Our first Adios compatible function
  int i = 0;

  // Write the text to the last line of the screen
  while(Text[i] != 0) {
    BltLetter(Text[i], (CurX * 8), 152, tFGC, tBGC);
    i++;
    CurX++;
    // If we've hit the end of the screen, we need to move the text up
    if (CurX == 30) {
      ShiftScreen();
      CurX = 0;
    }
  }
}

void PrintCR(unsigned char * Text)
{
  // Write the text to the last line of the screen
  Print(Text);
  // Now move the screen up to simulate sending a CRLF
  ShiftScreen();
  // Reset the cursor
  CurX = 0;
}
