//////////////////////////////////////////////////////////////////////////////
// GbaSioTool                                                               //
//////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <direct.h>
#include <stdio.h>
#include <conio.h>
#include "c:/dev/agb/include/AgbMemoryMap.h"
#include "c:/dev/agb/include/AgbDefine.h"

//////////////////////////////////////////////////////////////////////////////
// Defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
enum { TYPE, TYPE_DIR, TYPE_BMP, TYPE_PAL, TYPE_LZ77_BMP, TYPE_MB, TYPE_LOG };

//////////////////////////////////////////////////////////////////////////////
// Variables                                                                //
//////////////////////////////////////////////////////////////////////////////
HANDLE hCom;
FILE *fLog = 0;

//////////////////////////////////////////////////////////////////////////////
// ClearBuffers                                                             //
//////////////////////////////////////////////////////////////////////////////
void ClearBuffers()
{
	PurgeComm(hCom, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);	// purge any information in the buffer
}

//////////////////////////////////////////////////////////////////////////////
// RecvByte                                                                 //
//////////////////////////////////////////////////////////////////////////////
char RecvByte()
{
	char c = 0;
	DWORD read;
	ReadFile(hCom, &c, 1, &read, NULL);
	if (!read) ClearBuffers();
	return c;
}

//////////////////////////////////////////////////////////////////////////////
// RecvBuf                                                                  //
//////////////////////////////////////////////////////////////////////////////
void RecvBuf(void *buf, DWORD size)
{
	DWORD read;
	ReadFile(hCom, buf, 1, &read, NULL);
	if (read != size) ClearBuffers();
}

//////////////////////////////////////////////////////////////////////////////
// SendByte                                                                 //
//////////////////////////////////////////////////////////////////////////////
void SendByte(char c)
{
	DWORD written;
	WriteFile(hCom, &c, 1, &written, NULL);
	if (!written) ClearBuffers();
}

//////////////////////////////////////////////////////////////////////////////
// SendBuf                                                                  //
//////////////////////////////////////////////////////////////////////////////
void SendBuf(const void *buf, DWORD size)
{
	DWORD written;
	WriteFile(hCom, buf, size, &written, NULL);
	if (written != size) ClearBuffers();
}

//////////////////////////////////////////////////////////////////////////////
// SendString                                                               //
//////////////////////////////////////////////////////////////////////////////
void SendString(const char *s)
{
	while (*s) SendByte(*s++);
	SendByte(0);
}

//////////////////////////////////////////////////////////////////////////////
// RecvString                                                               //
//////////////////////////////////////////////////////////////////////////////
char *RecvString()
{
	static char s[256];
	char *p=s;
	while ((*p++ = RecvByte()) != 0);
	return s;
}

//////////////////////////////////////////////////////////////////////////////
// SendInt                                                                  //
//////////////////////////////////////////////////////////////////////////////
void SendInt(int i)
{
	SendByte((i>>0) & 255);
	SendByte((i>>8) & 255);
	SendByte((i>>16) & 255);
	SendByte((i>>24) & 255);
}

//////////////////////////////////////////////////////////////////////////////
// RecvInt                                                                  //
//////////////////////////////////////////////////////////////////////////////
int RecvInt()
{
	int i = RecvByte();
	i |= RecvByte() << 8;
	i |= RecvByte() << 16;
	i |= RecvByte() << 24;
	return i;
}

//////////////////////////////////////////////////////////////////////////////
// SendReg                                                                  //
//////////////////////////////////////////////////////////////////////////////
void SendReg(int reg, int value)
{
	SendByte('w');
	SendInt(reg);
	SendInt(value);
}

//////////////////////////////////////////////////////////////////////////////
// RecvReg                                                                  //
//////////////////////////////////////////////////////////////////////////////
int RecvReg(int reg)
{
	SendByte('w');
	SendInt(reg);
	return RecvInt();
}

//////////////////////////////////////////////////////////////////////////////
// IsInCommandMode                                                          //
//////////////////////////////////////////////////////////////////////////////
bool IsInCommandMode()
{
	ClearBuffers();
	Sleep(100);
	ClearBuffers();
	SendByte('?');
	for (int i=0; i<10; i++)
	{
		if (RecvByte() == '!') return true;
	}
	printf("Not in command mode! Last transmission might have failed.\n");
	return false;
}

//////////////////////////////////////////////////////////////////////////////
// UploadFile                                                               //
//////////////////////////////////////////////////////////////////////////////
int UploadFile(char *filename)//, bool bSendHandle)
{
	SendByte('u');

	FILE *f = fopen(filename, "rb");
	//if (bSendHandle) SendInt((int)f);

	if (!f) { printf("Error opening '%s' for input!\n", filename); return -1; }
	printf("Sending '%s'...\n", filename);

	// send size
	fseek(f, 0, SEEK_END);
	int size = ftell(f);
	SendInt(size);
	fseek(f, 0, SEEK_SET);

	// send file data
	char *buf = new char[size];
	fread(buf, size, 1, f);
	fclose(f);
	SendBuf(buf, size);
	delete buf;
	printf("Transfer complete\n");

	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// DownloadFile                                                             //
//////////////////////////////////////////////////////////////////////////////
int DownloadFile(char *filename)
{
	SendByte('d');

	FILE *f = fopen(filename, "wb");

	if (!f) { printf("Error opening '%s' for output!\n", filename); return -1; }
	printf("Receiving '%s'...\n", filename);

	// receive size
	int size = RecvInt();

	// receive file data
	char *buf = new char[size];
	fclose(f);
	RecvBuf(buf, size);
	fwrite(buf, size, 1, f);
	delete buf;
	printf("Transfer complete\n");

	return 0;
}

//////////////////////////////////////////////////////////////////////////////
// atexit1                                                                  //
//////////////////////////////////////////////////////////////////////////////
void atexit1()
{
	// close com port
	CloseHandle(hCom);
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	// open port
	hCom = CreateFile("COM1", GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL/*|FILE_TYPE_OVERLAPPED*/, NULL);
	if (!hCom) { printf("Error opening COM port!\n"); return -1; }

	// set port settings
	DCB dcb;
	GetCommState(hCom, &dcb);
	dcb.BaudRate = CBR_115200;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fOutxDsrFlow = FALSE;
	dcb.fDtrControl = DTR_CONTROL_ENABLE;
	dcb.fOutxCtsFlow = TRUE;
	dcb.fRtsControl = RTS_CONTROL_HANDSHAKE; 
	dcb.fInX = dcb.fOutX = FALSE;
	SetCommState(hCom, &dcb);

	// set timeouts
	COMMTIMEOUTS to;
	GetCommTimeouts(hCom, &to);
	to.ReadIntervalTimeout = 1000/30;
	to.ReadTotalTimeoutMultiplier = 1;
	to.ReadTotalTimeoutConstant = 1000/30;
	to.WriteTotalTimeoutMultiplier = 1;
	to.WriteTotalTimeoutConstant = 1000/30;
	SetCommTimeouts(hCom, &to);

	// init buffers
	SetupComm(hCom, 4096, 4096);	// setup device buffers
	ClearBuffers();

	atexit(atexit1);



	// change / show current directory
	bool exit = false;
	int type = 0;
	for (int i=1; i<argc; i++)
	{
		if (argv[i][0] == '-')
		{
			for (int j=1; argv[i][j]; j++)
			{
				switch (argv[i][j])
				{
					case 'd': type = TYPE_DIR; break;
					case 'b': type = TYPE_BMP; break;
					case 'p': type = TYPE_PAL; break;
					case 'z': type = TYPE_LZ77_BMP; break;
					case 'x': type = TYPE_MB; break;
					case 'l': type = TYPE_LOG; break;
					default: printf("Unknown option: '%c'!\n", argv[i][j]); break;
				}
			}
		}
		else
		{
			// assume type
			if (!type) if (strstr(argv[i], ".mb") || strstr(argv[i], ".ram")) type = TYPE_MB;
			if (!type) if (strstr(argv[i], "pal")) type = TYPE_PAL;
			if (!type) if (strstr(argv[i], "_LZ")) type = TYPE_LZ77_BMP;
			if (!type) if (strstr(argv[i], "bmp")) type = TYPE_BMP;
			//if (!type) if (strstr(argv[i], "log")) type = TYPE_LOG;

			switch (type)
			{
				case TYPE_DIR:
				{
					if (_chdir(argv[i]))
					{
						printf("Invalid directory: %s\n", argv[i]);
					}
					break;
				}

				case TYPE_BMP:
				{
					if (!IsInCommandMode()) return -1;
					UploadFile(argv[i]);
					if (!IsInCommandMode()) return -1;
					SendByte('b');
					exit = true;
					break;
				}

				case TYPE_PAL:
				{
					if (!IsInCommandMode()) return -1;
					UploadFile(argv[i]);
					if (!IsInCommandMode()) return -1;
					SendByte('p');
					SendReg(REG_DISPCNT, DISP_MODE_4 | DISP_BG2_ON);
					exit = true;
					break;
				}

				case TYPE_LZ77_BMP:
				{
					if (!IsInCommandMode()) return -1;
					UploadFile(argv[i]);
					if (!IsInCommandMode()) return -1;
					SendByte('z');
					exit = true;
					break;
				}

				case TYPE_MB:
				{
					if (!IsInCommandMode()) return -1;
					UploadFile(argv[i]);
					if (!IsInCommandMode()) return -1;
					SendByte('x');
					break;
				}

				case TYPE_LOG:
				{
					fLog = fopen(argv[i], "wt");
					if (!fLog) { printf("Error opening log file!\n"); }
					break;
				}

				default:
				{
					printf("Unknown file type: '%s'\n", argv[i]);
					return -1;
				}
			}
			
			type = 0;
		}
	}

	if (exit) return 0;

	//char s[256];
	//printf("Current directory is '%s'\n", _getcwd(s, 256));

	// get characters/commands
	while (1)
	{
		// transmit characters
		if (_kbhit())
		{
			char c = _getch();
			if (c == 27) break;		// escape
			SendByte(c);
		}

		// receive characters
		char c = RecvByte();
		{
			switch (c)
			{
				case 0:		// garbage | time-out
				case 255:	// garbage
				{
					break;
				}

				case '':	// file request
				{
					UploadFile(RecvString());
					break;
				}

				case '':	// accept file
				{
					DownloadFile(RecvString());
					break;
				}

				default:
				{
					if (fLog) fputc(c, fLog);
					printf("%c", c);
					break;
				}
			}
		}
	}

	if (fLog) fclose(fLog);
	return 0;
}
