//#include "../kernapi.h"
#include "text.h"
#include <stdio.h>

// A couple of helper functions to convert strings into numbers
// These should get round any endian problems
unsigned short ctoui16(unsigned char * Buf)
{
	return Buf[0] * 256 + Buf[1];
}

unsigned int ctoui32(unsigned char * Buf)
{
	return Buf [0] * 0x01000000
		 + Buf [1] * 0x00010000
		 + Buf [2] * 0x00000100
		 + Buf [3];
}

// Three functions to turn an integer into a hex string
char * ui8tohex(unsigned char n, unsigned char * s)
{
	int x;
	s[2] = 0;
	for(x = 1; x > -1; x--){
		switch ((n & 0x000F)){
		case 0: {
			s[x] = '0';
			break;}
		case 1: {
			s[x] = '1';
			break;}
		case 2: {
			s[x] = '2';
			break;}
		case 3: {
			s[x] = '3';
			break;}
		case 4: {
			s[x] = '4';
			break;}
		case 5: {
			s[x] = '5';
			break;}
		case 6:	{
			s[x] = '6';
			break;}
		case 7: {
			s[x] = '7';
			break;}
		case 8: {
			s[x] = '8';
			break;}
		case 9: {
			s[x] = '9';
			break;}
		case 10: {
			s[x] = 'A';
			break;}
		case 11: {
			s[x] = 'B';
			break;}
		case 12: {
			s[x] = 'C';
			break;}
		case 13: {
			s[x] = 'D';
			break;}
		case 14: {
			s[x] = 'E';
			break;}
		case 15: {
			s[x] = 'F';
			break;}
		}
		n = n >> 4;
	}
	
	return s;
}

char * ui16tohex(unsigned short n, unsigned char * s)
{
	int x;
	s[4] = 0;
	for(x = 3; x > -1; x--){
		switch ((n & 0x000F)){
		case 0: {
			s[x] = '0';
			break;}
		case 1: {
			s[x] = '1';
			break;}
		case 2: {
			s[x] = '2';
			break;}
		case 3: {
			s[x] = '3';
			break;}
		case 4: {
			s[x] = '4';
			break;}
		case 5: {
			s[x] = '5';
			break;}
		case 6:	{
			s[x] = '6';
			break;}
		case 7: {
			s[x] = '7';
			break;}
		case 8: {
			s[x] = '8';
			break;}
		case 9: {
			s[x] = '9';
			break;}
		case 10: {
			s[x] = 'A';
			break;}
		case 11: {
			s[x] = 'B';
			break;}
		case 12: {
			s[x] = 'C';
			break;}
		case 13: {
			s[x] = 'D';
			break;}
		case 14: {
			s[x] = 'E';
			break;}
		case 15: {
			s[x] = 'F';
			break;}
		}
		n = n >> 4;
	}
	
	return s;
}

char * ui32tohex(unsigned int n, unsigned char * s)
{
	int x;
	s[8] = 0;
	for(x = 7; x > -1; x--){
		switch ((n & 0x0000000F)){
		case 0: {
			s[x] = '0';
			break;}
		case 1: {
			s[x] = '1';
			break;}
		case 2: {
			s[x] = '2';
			break;}
		case 3: {
			s[x] = '3';
			break;}
		case 4: {
			s[x] = '4';
			break;}
		case 5: {
			s[x] = '5';
			break;}
		case 6: {
			s[x] = '6';
			break;}
		case 7: {
			s[x] = '7';
			break;}
		case 8: {
			s[x] = '8';
			break;}
		case 9: {
			s[x] = '9';
			break;}
		case 10: {
			s[x] = 'A';
			break;}
		case 11: {
			s[x] = 'B';
			break;}
		case 12: {
			s[x] = 'C';
			break;}
		case 13: {
			s[x] = 'D';
			break;}
		case 14: {
			s[x] = 'E';
			break;}
		case 15: {
			s[x] = 'F';
			break;}
		}
		n = n >> 4;
	}
	
	return s;
}

// Two functions to check if a particular bit in a value is set
int CheckBit16(unsigned short Value, int Bit)
{
	return (int)((Value >> Bit) & 0x0001);
}

int CheckBit32(unsigned int Value, int Bit)
{
	return (int)((Value >> Bit) & 0x00000001);
}

// Two functions to convert integers into strings ready to send
unsigned char * ui16toc(unsigned short n, unsigned char * s)
{
	s[0] = (unsigned char)(n >> 8) & 0xFF;
	s[1] = (unsigned char)(n & 0xFF);
	return s;
}

unsigned char * ui32toc(unsigned int n, unsigned char * s)
{
	s[0] = (unsigned char)(n >> 24) & 0xFF;
	s[1] = (unsigned char)(n >> 16) & 0xFF;
	s[2] = (unsigned char)(n >> 8) & 0xFF;
	s[3] = (unsigned char)(n & 0xFF);
	return s;
}

// Same as memcopy, but without the library
void ByteCopy(register unsigned char * Dest, register unsigned char * Source, register int len)
{
	register int n;
	for(n = 0; n < len; n++) {
		*Dest = *Source;
		Dest++;
		Source++;
	}
}
/*void ByteCopy(unsigned char * Dest, unsigned char * Source, int len)
{
	int n;
	for(n = 0; n < len; n++) {
		Dest[n] = Source[n];
	}
}*/

// Dump raw data to the screen as hex
void DumpRaw(unsigned char * Data, int len)
{
	int n;
	char Hex[3];
	char ScreenBuf[256];

	Print("Dump:");
	for(n = 0; n < len; n++){
		sprintf(ScreenBuf, " [%s]", ui8tohex(Data[n], Hex));
		Print(ScreenBuf);
	} PrintCR("");
}
